import os


class BaseRenderer(object):

    def render(self, template, values):
        raise NotImplementedError


class DjangoRenderer(BaseRenderer):

    def __init__(self, path):
        from django.conf import settings
        from django.template.loader import render_to_string

        path = os.path.abspath(path)
        settings.configure(DEBUG=True, TEMPLATE_DEBUG=True,
                           TEMPLATE_DIRS=[path])
        self._render = render_to_string

    def render(self, template, values):
        return self._render(template, values)


class Jinja2Renderer(BaseRenderer):

    def __init__(self, path):
        from jinja2 import Environment, FileSystemLoader
        self.env = Environment(loader=FileSystemLoader(path))
        self.env.filters['in_category'] = self.is_in_category

    def render(self, template, values):
        t = self.env.get_template(template)
        return t.render(values)

    @classmethod
    def is_in_category(cls, post, category):
        slugs = [c['slug'] for c in post.categories]
        return category in slugs



import os
import hashlib
from datetime import datetime
import yaml
from .processors import RstProcessor, MarkdownProcessor, TextileProcessor
from .utils import slugify, ligatures
from .exceptions import ConfigurationError
from .typography import typogrify

EXTENSIONS = {
    '.md': 'markdown',
    '.markdown': 'markdown',
    '.mkdn': 'markdown',
    '.rst': 'rst',
    '.html': 'html',
    '.htm': 'html',
    '.txt': 'html',
    '.textile': 'textile'
}


class File(object):

    def __init__(self, path, context):

        self.context = context  # site wide config
        self.path = path

        self.filename = os.path.basename(path)
        self.parse()

        # The file should have at least a title
        try:
            self.title = self.config['title']
        except KeyError:
            raise ConfigurationError

        if self.context['punctuation']:
            self.contents = typogrify(self.contents)

        if self.context['ligatures']:
            self.contents = ligatures(self.contents)

    def hash_file(self):
        return hashlib.md5(open(self.path).read()).hexdigest()

    def _get_type(self):
        name, extension = os.path.splitext(self.path)
        try:
            self.file_type = EXTENSIONS[extension]
            return self.file_type
        except KeyError:
            raise Exception('Unknown extension')

    def parse(self):
        """
        Decide what file type the current file is.
        """
        is_rst = False
        if self.context['text_processor'] == 'extension':
            file_type = self._get_type()
            is_rst = file_type == 'rst'

        if self.context['text_processor'] in ['markdown', 'textile', 'html']:
            is_rst = False
            self.file_type = self.context['text_processor']

        if is_rst:
            self._parse_rst()
        else:
            self._parse()

    def _parse(self):
        """
        Read file contents. Extract yaml config. Return post contents and
        parsed config.
        """
        f = open(self.path, 'r')

        c = ""
        start = False
        end = False
        conf = ""
        for x in f:
            if x.startswith('-' * 79):
                if not start:
                    start = True
                else:
                    end = True
                continue

            if start and not end:
                conf += x
            else:
                c += x
        f.close()

        self.config = yaml.load(conf)
        self.contents = self._process_contents(c)

    def _parse_rst(self):
        try:
            h = self.context['initial_header_level']
        except KeyError:
            h = 2
        p = RstProcessor(self.path, self.context, 'html', h)
        self.contents = p.content
        self.config = p.metadata

        try:
            d = datetime.strptime(self.config['date'], '%Y-%m-%d %H:%M')
        except ValueError:
            try:
                d = datetime.strptime(self.config['date'], '%Y-%m-%d %H:%M:%S')
            except ValueError:
                raise ConfigurationError

        self.year = d.year
        self.month = d.strftime("%m")
        self.date = d.strftime(self.context['date_format'])
        try:
            self.config['categories'] = self.config['categories'].split(', ')
        except KeyError:
            pass

        self.config['date'] = d

    def _process_contents(self, text):
        """
        Run contents through a text processor.
        Options:
            - markdown
            - textile
            - html
        """

        text = str(text, "utf-8")

        p = self.file_type
        p = p.lower()

        if p == 'markdown':
            processor = MarkdownProcessor()
            html = processor.render(text)
        elif p == 'textile':
            processor = TextileProcessor()
            html = processor.render(text)
        elif p == 'html':
            html = text
        else:
            raise Exception("Unknown text processor")

        return html


class Post(File):
    """
    Post object. Exposes the following attributes:
        - filename (e.g. 2010-cool-post.md)
        - path (e.g. /home/user/2010-cool-post.md)
        - contents (post content)
        - config (post front config)
        - year, month, date (in the format specified in config.yaml)
        - title
        - url (e.g. /2010/02/cool-post/)
        - categories (list)
        - author (from config.yaml or from front config)

    All attributes are calculated on init.
    """

    def __init__(self, path, context):
        super(Post, self).__init__(path, context)

        self.year = str(self.config['date'].year)
        self.month = self.config['date'].strftime("%m")
        self.day = self.config['date'].strftime("%d")
        self.date = self.config['date'].strftime(self.context['date_format'])
        self.atom_date = self._get_atom_date(self.config['date'])
        if self.config.get('slug'):
            self.slug = self.config['slug']
        else:
            self.slug = slugify(self.config['title'])

        if context['append_slash']:
            url_template = '%s/%s/%s/'
        else:
            url_template = '%s/%s/%s'

        if context['url_include_day']:
            url_template = '%s/' + url_template
            self.url = url_template % (self.year, self.month, self.day,
                                       self.slug,)
        else:
            self.url = url_template % (self.year, self.month, self.slug,)

        categories = self.config.get('categories', ['Uncategorized'])
        self.categories = []
        for c in categories:

            if not c:
                continue

            v = {
                'name': c,
                'slug': slugify(c)
            }
            self.categories.append(v)

        if 'author' not in self.config:
            self.author = self.context['author']
        else:
            self.author = self.config['author']

        self.reading_time = self._get_reading_time()

    def _get_atom_date(self, date):
        d = date.strftime('%Y-%m-%dT%H:%M:%S%z')
        return d + "Z"

    def _get_reading_time(self):
        words = len(self.contents.split(' '))
        reading_time = int(float(words) / 200.0)

        if reading_time < 1:
            return 'under one minute'
        elif reading_time == 1:
            return 'about one minute'
        else:
            return '%d minutes' % reading_time


class Page(File):

    def __init__(self, path, context):
        super(Page, self).__init__(path, context)
        if self.config.get('slug'):
            self.slug = self.config['slug']
        else:
            self.slug = slugify(self.config['title'])



import docutils

import docutils.core
from docutils.writers.html4css1 import HTMLTranslator
from docutils.writers.latex2e import LaTeXTranslator
from docutils import nodes
from docutils.parsers.rst import directives, Directive

from pygments import highlight
from pygments.lexers import get_lexer_by_name, TextLexer
from pygments.formatters import HtmlFormatter
from pygments.formatters import LatexFormatter

from misaka import HtmlRenderer, Markdown, EXT_FENCED_CODE
from textile import textile

# Set to True if you want inline CSS styles instead of classes
INLINESTYLES = False

# The default formatter
DEFAULT = LatexFormatter()
DEFAULT = HtmlFormatter(noclasses=INLINESTYLES)

# Add name -> formatter pairs for every variant you want to use
VARIANTS = {
    # 'linenos': HtmlFormatter(noclasses=INLINESTYLES, linenos=True),
}


def raw_html(name, args, options, content, lineno, contentOffset, blockText,
             state, stateMachine):
    """
    Simply render the input html as html.
    """
    if content == "":
        return
    return [nodes.raw(text='\n'.join(content), format='html')]


raw_html.content = True
directives.register_directive('raw_html', raw_html)


class Pygments(Directive):
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = dict([(key, directives.flag) for key in VARIANTS])
    has_content = True
    formatter = DEFAULT

    def run(self):
        self.assert_has_content()
        try:
            lexer = get_lexer_by_name(self.arguments[0])
        except ValueError:
            # no lexer found - use the text one instead of an exception
            lexer = TextLexer()
        # take an arbitrary option if more than one is given
        if self.options and VARIANTS[list(self.options.keys())[0]]:
            self.formatter = VARIANTS[list(self.options.keys())[0]]

        parsed = highlight('\n'.join(self.content), lexer, self.formatter)
        return [nodes.raw('', parsed, format='html')]


class Processor(object):
    """
    Processor subclass. Markdown, textile etc processors should inherit
    from this and override the ``render(text)`` method

    Note: ``RstProcessor`` works completely differently and isn't a subclass of
    ``Processor`` at this time.
    """

    def render(self, text):
        raise NotImplementedError


class RstProcessor(object):

    allowed_types = ['html', 'latex', 'xetex']

    class HtmlTranslator(HTMLTranslator):
        def astext(self):
            return ''.join(self.body)

        def visit_field_body(self, node):
            pass

        def depart_field_body(self, node):
            pass

    class LatexTranslator(LaTeXTranslator):
        def astext(self):
            return ''.join(self.body)

        def visit_field_body(self, node):
            pass

        def depart_field_body(self, node):
            pass

    def __init__(self, filename, global_settings, output='html',
                 header_level=2):

        self.settings = global_settings
        self.pygments_builder()

        directives.register_directive('sourcecode', self.pygmenter)
        directives.register_directive('code-block', self.pygmenter)

        self.filename = filename
        if output not in self.allowed_types:
            raise NotImplementedError("Can't render '%s'." % output)
        self.header_level = str(header_level)
        self.output = output
        self.metadata = None
        self.get_publisher()
        self.get_metadata()
        self.run()

    def pygments_builder(self):

        self.pygmenter = Pygments
        if self.settings['pygments']:
            self.pygmenter.formatter = HtmlFormatter(
                **self.settings['pygments'])

    def render_node_to_html(self, document, node):
        if self.output == 'html':
            visitor = self.HtmlTranslator(document)
        elif self.output == 'latex' or self.output == 'xetex':
            visitor = self.LatexTranslator(document)
        else:
            pass

        node.walkabout(visitor)
        return visitor.astext()

    def get_metadata(self):
        self.metadata = {}
        for docinfo in self.pub.document.traverse(docutils.nodes.docinfo):
            for element in docinfo.children:
                if element.tagname == 'field':
                    name_elem, body_elem = element.children
                    name = name_elem.astext()
                    value = self.render_node_to_html(self.pub.document,
                                                     body_elem)
                else:
                    name = element.tagname
                    value = element.astext()
                self.metadata[name] = value

    def get_publisher(self):
        extra_params = {
            'initial_header_level': self.header_level
        }
        pub = docutils.core.Publisher(
            destination_class=docutils.io.StringOutput)
        pub.set_components('standalone', 'restructuredtext', self.output)
        pub.process_programmatic_settings(None, extra_params, None)
        pub.set_source(source_path=self.filename)
        pub.publish()
        self.pub = pub

    def run(self):
        parts = self.pub.writer.parts
        if self.output == 'html':
            content = parts.get('body')
        elif self.output == 'latex' or self.output == 'xetex':
            content = parts.get('body')
            content = parts.get('whole')
        else:
            pass

        self.content = content


class MarkdownProcessor(Processor):

    class MisakaProcessor(HtmlRenderer):

        def block_code(self, text, lang):
            return self.highlight_code(lang, text)

        def highlight_code(self, language, code):
            return highlight(code, get_lexer_by_name(language),
                             HtmlFormatter())

    def __init__(self):
        self.md = Markdown(self.MisakaProcessor(), EXT_FENCED_CODE)

    def render(self, text):
        return self.md(text)


class TextileProcessor(Processor):

    def render(self, text):
        return textile(text)



from setuptools import setup

description = 'Socrates is a simple static site generator.'
long_desc = open('README.rst').read()

setup(
    name='socrates',
    version='0.9.3',
    url='http://honza.github.com/socrates/',
    install_requires=[
        'PyYAML==3.11',
        'misaka==2.0.0',
        'Django==1.8.6',
        'Jinja2==2.8',
        'docutils==0.12',
        'Pygments==2.0.2',
        'textile==2.2.2'
    ],
    description=description,
    long_description=long_desc,
    author='Honza Pokorny',
    author_email='me@honza.ca',
    maintainer='Honza Pokorny',
    maintainer_email='me@honza.ca',
    packages=['socrates'],
    include_package_data=True,
    scripts=['bin/socrates'],
)



"""
Socrates - Static site generator
(c) 2011-2015 - Honza Pokorny et al.
http://github.com/honza/socrates
BSD licensed
"""
import os
import sys
import shutil
import yaml
import json
from datetime import datetime

from .renderers import DjangoRenderer, Jinja2Renderer
from .models import Post, Page
from .utils import slugify
from .exceptions import ConfigurationError


DEFAULTS = {
    'author': 'author',
    'site_name': 'Socrates site',
    'posts_per_page': 10,
    'url': 'http://example.com',
    'date_format': '%B %d, %Y',
    'text_processor': 'markdown',
    'templates': 'django',
    'append_slash': True,
    'url_include_day': False,
    'initial_header_level': 2,
    'skip_archives': False,
    'skip_categories': False,
    'inline_css': False,
    'pygments': {},
    'sitemap_urls': ['about'],
    'punctuation': False,
    'ligatures': False,
    'deploy_dir': 'deploy'
}


AVAILABLE_EXTENSIONS = ['markdown', 'rst', 'textile', 'html', 'extension']
POST_CACHE_FILENAME = '.post-cache.json'


class Generator(object):
    """
    This is the main class of the application. It handles the collection,
    processing and rendering of the site.
    """

    # Templates
    SINGLE = 'single.html'
    PAGE = 'page.html'
    INDEX = 'index.html'
    CATEGORY = 'category.html'
    ARCHIVE = 'archive.html'
    PAGED = 'index_paged.html'
    ATOM = 'atom.html'
    SITEMAP = 'sitemap.xml'

    # Directories
    ROOT = None
    DEPLOY = None
    POSTS = None
    PAGES = None

    # Global, site-wide settings
    SETTINGS = None

    def __init__(self, directory, silent=False):
        m = os.getcwd()
        self.ROOT = os.path.join(m, directory)
        self.silent = silent

        if not os.path.exists(self.ROOT):
            sys.stderr.write("The '%s' directory doesn't exist.\n" % directory)
            return
        self.SETTINGS = self._get_settings()
        # Set up deploy directory
        self.DEPLOY = os.path.join(self.ROOT, self.SETTINGS['deploy_dir'])
        self.deploy_dir_created = False
        if not os.path.exists(self.DEPLOY):
            os.mkdir(self.DEPLOY)
            self.deploy_dir_created = True
        # Copy media files to deploy destination
        if os.path.exists(os.path.join(self.DEPLOY, 'media')):
            shutil.rmtree(os.path.join(self.DEPLOY, 'media'))
        shutil.copytree(os.path.join(self.ROOT, 'layout', 'media'),
                        os.path.join(self.DEPLOY, 'media'))

        self.POSTS = os.path.join(self.ROOT, 'posts')
        self.PAGES = os.path.join(self.ROOT, 'pages')

        if self.SETTINGS['text_processor'] not in AVAILABLE_EXTENSIONS:
            ext = self.SETTINGS['text_processor']
            sys.stderr("WARNING: %s isn't a recognizeed text processor.\n"
                       % ext)
            sys.exit(1)
        self.init_template_renderer()

        self.posts = []
        self.pages = []
        self.categories = {}
        self.years = {}
        self.archives = {}

        self.load_posts()
        self.cache_data = self.get_post_cache()
        self.process_posts()

        if os.path.exists(self.PAGES):
            self.load_pages()
            self.process_pages()

        self.SETTINGS = dict(self.SETTINGS, **{'categories': self.categories})
        self.SETTINGS = dict(self.SETTINGS, **{'years': self.years})

        # Create html files
        self.make_index_page()
        self.make_atom()
        self.make_sitemap()

        if not self.SETTINGS['skip_categories']:
            self.make_category_pages()
        if not self.SETTINGS['skip_archives']:
            self.make_archive_pages()

        self.make_pagination()

        if not self.SETTINGS['inline_css']:
            from pygments.formatters import HtmlFormatter
            formatter = HtmlFormatter(**self.SETTINGS['pygments'])
            css_content = formatter.get_style_defs('.highlight')
            self._write_to_file(
                os.path.join(self.DEPLOY, 'media', 'pygments.css'),
                css_content)

        self.save_post_cache()
        self.log("Success!")

    def init_template_renderer(self):
        """
        Create and save a reference to an instance of a template renderer
        """
        t = self.SETTINGS['templates']
        layout = os.path.join(self.ROOT, 'layout')
        if t == 'jinja2':
            self.template = Jinja2Renderer(layout)
        elif t == 'django':
            self.template = DjangoRenderer(layout)
        else:
            raise NotImplementedError("I don't know this template type.")

    def render(self, template, values):
        """
        Give a template name and a dict of values, return an HTML
        representation of a page/post.
        """
        return self.template.render(template, values)

    def log(self, text):
        """
        A thin wrapper around the ``print`` statement to allow silencing of all
        non-error output.
        """
        if not self.silent:
            print(text)

    def _get_page_str(self, current, total):
        """
        Return page number as a string for pagination.
        """
        current += 1
        if total < 10:
            return str(current)
        elif 10 <= total < 100:
            if total < 10:
                return '0%d' % current
            else:
                return str(current)

    def _get_post_cats(self, post):
        """
        Extract and save post's categories
        """
        cats = post.config.get('categories', ['Uncategorized'])
        for c in cats:
            if c not in self.categories:
                self.categories[c] = [post]
            else:
                self.categories[c].append(post)

    def _get_settings(self):
        """
        Read main config file. Return dict.
        """
        s = os.path.join(self.ROOT, 'config.yaml')
        if not os.path.exists(s):
            raise Exception('No config file.')
        c = open(s, 'r').read()
        return dict(DEFAULTS, **yaml.load(c))

    def _write_to_file(self, path, contents):
        """
        Create a file (path) with contents
        """
        f = open(path, 'w')
        f.write(contents.encode("utf-8"))
        f.close()

    def _v(self, vals):
        """
        Create a joint dict of global config and local template vals
        """
        return dict(self.SETTINGS, **vals)

    def load_posts(self):
        """
        Get all files from the posts directory, create Post instances and add
        them to the self.posts list.
        """
        filenames = os.listdir(self.POSTS)
        filenames.sort()

        for filename in filenames:
            if not filename.startswith('.') and not filename.startswith('_'):
                p = os.path.join(self.POSTS, filename)
                try:
                    self.posts.append(Post(p, self.SETTINGS))
                except ConfigurationError:
                    sys.stderr.write("WARNING: %s isn't configured properly.\n"
                                     % filename)
                    sys.exit(1)
                except:
                    sys.stderr.write("ERROR: couldn't parse %s.\n"
                                     % filename)
                    sys.exit(1)

        self.posts.reverse()

    def load_pages(self):
        """
        Get all files from the posts directory, create Post instances and add
        them to the self.posts list.
        """
        for filename in os.listdir(self.PAGES):
            if not filename.startswith('.') and not filename.startswith('_'):
                p = os.path.join(self.PAGES, filename)
                try:
                    self.pages.append(Page(p, self.SETTINGS))
                except ConfigurationError:
                    sys.stderr.write("WARNING: %s isn't configured properly.\n"
                                     % filename)
                    sys.exit(1)

    def process_posts(self):
        """
        Collect all the necessary information about posts.
        """
        self.log('Processing posts...')
        for post in self.posts:

            # Get categories
            self._get_post_cats(post)

            # Post dirs
            #   years
            date = post.config['date']
            year = str(date.year)
            if year not in self.years:
                self.years[year] = {}
            #   months
            m = date.strftime("%m")
            y = self.years[year]
            if m not in list(y.keys()):
                self.years[year][m] = []

            if self.SETTINGS['url_include_day']:
                if post.day not in self.years[year][m]:
                    self.years[year][m].append(post.day)

            # Archives
            d = year
            if d not in self.archives:
                self.archives[d] = [post]
            else:
                self.archives[d].append(post)

        self.make_post_directories()
        self.save_posts()

    def process_pages(self):
        self.save_pages()

    def save_posts(self):
        self.log('Saving posts...')
        for post in self.posts:

            # TODO: All of this caching needs to happen much sooner than here
            # because the posts have already been parsed.
            try:
                old_hash = self.cache_data[post.path]
            except KeyError:
                old_hash = None

            new_hash = post.hash_file()
            self.cache_data[post.path] = new_hash

            if old_hash == new_hash:
                continue

            # Save the thing
            b = post.slug + '.html'

            if self.SETTINGS['url_include_day']:
                m = os.path.join(self.DEPLOY, post.year, post.month,
                                 post.day, b)
            else:
                m = os.path.join(self.DEPLOY, post.year, post.month, b)

            if 'template' in list(post.config.keys()):
                t = post.config['template']
            else:
                t = self.SINGLE

            content = self.render(t, self._v({'post': post}))

            # Print filename to show progress
            self.log(post.filename)

            self._write_to_file(m, content)

    def save_pages(self):
        self.log('Saving pages...')
        for page in self.pages:
            # Save the thing
            b = page.slug + '.html'

            m = os.path.join(self.DEPLOY, b)

            if 'template' in list(page.config.keys()):
                t = page.config['template']
            else:
                t = self.PAGE

            content = self.render(t, self._v({'page': page}))

            # Print filename to show progress
            self.log(page.filename)
            self._write_to_file(m, content)

    def make_post_directories(self):
        """
        If they don't exist, create post directories.
            2011
                01
                  04
                  05
                  12
                03
                  09
                  11
                12
                  21
            2010
                04
                  11
                07
                  30
        """
        self.log('Creating directories...')
        keys = list(self.years.keys())
        for k in keys:
            m = os.path.join(self.DEPLOY, k)
            if not os.path.exists(m):
                os.mkdir(m)

            months = self.years[k]
            for month in months:
                m = os.path.join(self.DEPLOY, k, month)
                if not os.path.exists(m):
                    os.mkdir(m)

                if self.SETTINGS['url_include_day']:
                    days = self.years[k][month]
                    for day in days:
                        m = os.path.join(self.DEPLOY, k, month, day)
                        if not os.path.exists(m):
                            os.mkdir(m)

    def make_index_page(self):
        """
        Create an index page
        """
        m = os.path.join(self.DEPLOY, 'index.html')
        if self.SETTINGS['posts_per_page'] == 0:
            # List all posts on the index page
            posts = self.posts
            extra = False
        elif len(self.posts) > self.SETTINGS['posts_per_page']:
            # Cut off unnecessary posts
            n = len(self.posts)
            n = self.SETTINGS['posts_per_page']
            posts = self.posts[:n]
            extra = True
        else:
            posts = self.posts
            extra = False
        contents = self.render(self.INDEX,
                               self._v({'posts': posts, 'extra': extra}))
        self._write_to_file(m, contents)

    def make_sitemap(self):
        """
        Generate a sitemap.
        """
        m = os.path.join(self.DEPLOY, 'sitemap.xml')
        posts = self.posts
        contents = self.render(self.SITEMAP, self._v({
            'posts': posts,
            'now': self._get_atom_date()}))
        self._write_to_file(m, contents)

    def make_atom(self):
        """
        Create an atom feed
        """
        m = os.path.join(self.DEPLOY, 'atom.xml')
        if self.SETTINGS['posts_per_page'] == 0:
            # List all posts on the index page
            posts = self.posts
        elif len(self.posts) > self.SETTINGS['posts_per_page']:
            # Cut off unnecessary posts
            n = len(self.posts)
            n = self.SETTINGS['posts_per_page']
            posts = self.posts[:n]
        else:
            posts = self.posts
        contents = self.render(self.ATOM,
                               self._v({'posts': posts,
                                        'now': self._get_atom_date()}))

        contents = contents.replace('&nbsp;', '')
        self._write_to_file(m, contents)

    def _get_atom_date(self):
        date = datetime.utcnow()
        d = date.strftime('%Y-%m-%dT%H:%M:%S%z')
        return d + "Z"

    def make_category_pages(self):
        """
        Make category pages. They go into the 'category' directory.
        """
        keys = list(self.categories.keys())
        if len(keys) != 0:
            # Make category dir
            self.CATEGORIES = os.path.join(self.DEPLOY, 'category')
            if not os.path.exists(self.CATEGORIES):
                os.mkdir(self.CATEGORIES)
            for k in keys:
                if not k:
                    continue
                p = os.path.join(self.CATEGORIES, slugify(k))
                if not os.path.exists(p):
                    os.mkdir(p)
                posts = self.categories[k]
                contents = self.render(
                    self.CATEGORY, self._v({'category': k, 'posts': posts}))
                m = os.path.join(p, 'index.html')
                self._write_to_file(m, contents)

    def make_archive_pages(self):
        """
        Make archive pages. Only by year.
        """
        self.log('Creating archives...')
        keys = list(self.archives.keys())
        if len(keys) != 0:
            # Make category dir
            self.ARCHIVES = os.path.join(self.DEPLOY, 'archive')
            if not os.path.exists(self.ARCHIVES):
                os.mkdir(self.ARCHIVES)
            for k in keys:
                p = os.path.join(self.ARCHIVES, slugify(k))
                if not os.path.exists(p):
                    os.mkdir(p)
                posts = self.archives[k]
                contents = self.render(
                    self.ARCHIVE, self._v({'year': k, 'posts': posts}))
                m = os.path.join(p, 'index.html')
                self._write_to_file(m, contents)

    def make_pagination(self):
        """
        Make pagination.
            /page/2/
            /page/3/
        Uses posts per page setting
        """
        per = int(self.SETTINGS['posts_per_page'])
        if per == 0:
            # Skip pagination if all posts are on index page
            return
        self.log('Creating pagination...')
        num = len(self.posts)
        pages = num / per

        m = os.path.join(self.DEPLOY, 'page')
        if not os.path.exists(m):
            os.mkdir(m)
        for x in range(1, pages + 1):
            if x == 1:
                prev = '/'
            else:
                prev = '/page/%d/' % x

            if x < pages:
                next = '/page/%d/' % int(x + 1)
            else:
                next = None

            n = per * x
            p = self.posts[n:n + per]

            v = {
                'page': x + 1,
                'posts': p,
                'prev': prev,
                'total': pages + 1,
                'next': next
            }

            folder = self._get_page_str(x, pages + 1)

            e = os.path.join(m, folder)
            if not os.path.exists(e):
                os.mkdir(e)

            contents = self.render(self.PAGED, self._v(v))
            c = os.path.join(e, "index.html")
            self._write_to_file(c, contents)

    def get_post_cache(self):
        if not os.path.exists(POST_CACHE_FILENAME) or self.deploy_dir_created:
            return {}

        return json.loads(open(POST_CACHE_FILENAME).read())

    def save_post_cache(self):
        self.log("Saving cache")
        with open(POST_CACHE_FILENAME, 'w') as f:
            f.write(json.dumps(self.cache_data, indent=4))



import re


def slugify(value):
    value = str(re.sub('[^\w\s-]', '', value).strip().lower())
    return re.sub('[-\s]+', '-', value)


def ligatures(text):
    # TODO: Georgia doesn't like the following three.
    # text = text.replace('ft', '&#xFB05;')
    # text = text.replace('ffl', '&#xFB04;')
    # text = text.replace('ffi', '&#xFB03;')
    text = text.replace('fl', '&#xFB02;')
    text = text.replace('fi', '&#xFB01;')
    return text
